-- Striegel specialization for mechanical weed management
--
-- only usefull in combination with multiSprayer/Herbicide-Mod by upsidedown
-- check http://www.modhoster.de/mods/multisprayer-herbizid-mod
--
--
-- removes not fully grown weed mechanically
--
-- 99% based on cultivator-specialisation by Stefan Geiger/Giants software
--
-- *** It is explicitely allowed to use this specialisation in your own mods ***
-- for changes please contact upsidedown at modhoster.de
--
-- V1.0:
-- start: 24.04.2014
-- finished: 29.04.2014
-- Author: upsidedown


Striegel = {}
--source("StriegelAreaEvent.lua")
function Striegel.prerequisitesPresent(specializations)
  return true
end
function Striegel:load(xmlFile)
  self.groundContactReport = SpecializationUtil.callSpecializationsFunction("groundContactReport")
  self.speedRotatingParts = {}
  local i = 0
  while true do
    do
      local baseName = string.format("vehicle.speedRotatingParts.speedRotatingPart(%d)", i)
      local index = getXMLString(xmlFile, baseName .. "#index")
      if index == nil then
        break
      end
      local node = Utils.indexToObject(self.components, index)
      if node ~= nil then
        local entry = {}
        entry.node = node
        entry.rotationSpeedScale = getXMLFloat(xmlFile, baseName .. "#rotationSpeedScale")
        if entry.rotationSpeedScale == nil then
          entry.rotationSpeedScale = 1 / Utils.getNoNil(getXMLFloat(xmlFile, baseName .. "#radius"), 1)
        end
        entry.rotateOnGroundContact = Utils.getNoNil(getXMLBool(xmlFile, baseName .. "#rotateOnGroundContact"), false)
        entry.foldMinLimit = Utils.getNoNil(getXMLFloat(xmlFile, baseName .. "#foldMinLimit"), 0)
        entry.foldMaxLimit = Utils.getNoNil(getXMLFloat(xmlFile, baseName .. "#foldMaxLimit"), 1)
        table.insert(self.speedRotatingParts, entry)
      end
      i = i + 1
    end
  end
  local drumNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.drum#index"))
  if drumNode ~= nil then
    print("Warning: vehicle.drum is no longer used, use speedRotatingParts\n")
  end
  self.contactReportNodes = {}
  local contactReportNodeFound = false
  local i = 0
  while true do
    do
      local baseName = string.format("vehicle.contactReportNodes.contactReportNode(%d)", i)
      local index = getXMLString(xmlFile, baseName .. "#index")
      if index == nil then
        break
      end
      local node = Utils.indexToObject(self.components, index)
      if node ~= nil then
        local entry = {}
        entry.node = node
        entry.hasGroundContact = false
        self.contactReportNodes[node] = entry
        contactReportNodeFound = true
      end
      i = i + 1
    end
  end
  if not contactReportNodeFound then
    local entry = {}
    entry.node = self.components[1].node
    entry.hasGroundContact = false
    self.contactReportNodes[entry.node] = entry
  end
  self.groundReferenceThreshold = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.groundReferenceNode#threshold"), 0.2)
  self.groundReferenceNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.groundReferenceNode#index"))
  local StriegelSound = getXMLString(xmlFile, "vehicle.StriegelSound#file")
  self.StriegelSoundEnabled = false
  if StriegelSound ~= nil and StriegelSound ~= "" then
    StriegelSound = Utils.getFilename(StriegelSound, self.baseDirectory)
    self.StriegelSound = createSample("StriegelSound")
    loadSample(self.StriegelSound, StriegelSound, false)
    self.StriegelSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.StriegelSound#pitchOffset"), 0)
    self.StriegelSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.StriegelSound#volume"), 1)
    setSamplePitch(self.StriegelSound, self.StriegelSoundPitchOffset)
  end
  self.StriegelDirectionNode = Utils.getNoNil(Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.StriegelDirectionNode#index")), self.components[1].node)
  self.groundParticleSystems = {}
  local entry = {}
  entry.ps = {}
  Utils.loadParticleSystem(xmlFile, entry.ps, "vehicle.groundParticleSystem", self.components, false, nil, self.baseDirectory)
  if 0 < table.getn(entry.ps) then
    entry.isActive = false
    table.insert(self.groundParticleSystems, entry)
  end
  local i = 0
  while true do
    do
      local baseName = string.format("vehicle.groundParticleSystems.groundParticleSystem(%d)", i)
      if not hasXMLProperty(xmlFile, baseName) then
        break
      end
      local entry = {}
      entry.ps = {}
      Utils.loadParticleSystem(xmlFile, entry.ps, baseName, self.components, false, nil, self.baseDirectory)
      if 0 < table.getn(entry.ps) then
        entry.isActive = false
        entry.cuttingArea = i + 1
        table.insert(self.groundParticleSystems, entry)
      end
      i = i + 1
    end
  end
  self.onlyActiveWhenLowered = Utils.getNoNil(getXMLBool(xmlFile, "vehicle.onlyActiveWhenLowered#value"), true)
  self.aiTerrainDetailChannel1 = g_currentMission.ploughChannel
  self.aiTerrainDetailChannel2 = g_currentMission.sowingChannel
  self.aiTerrainDetailChannel3 = g_currentMission.sowingWidthChannel
  self.maxSpeedLevel = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.maxSpeedLevel#value"), 1)
  self.speedViolationMaxTime = 2500
  self.speedViolationTimer = self.speedViolationMaxTime
  self.StriegelContactReportsActive = false
  self.startActivationTimeout = 2000
  self.startActivationTime = 0
  self.StriegelHasGroundContact = false
  self.StriegelLimitToField = false
  self.StriegelForceLimitToField = true
  self.showFieldNotOwnedWarning = false
  self.StriegelGroundContactFlag = self:getNextDirtyFlag()
end
function Striegel:delete()
  for _, entry in ipairs(self.groundParticleSystems) do
    Utils.deleteParticleSystem(entry.ps)
    entry.isActive = false
  end
  Striegel.removeContactReports(self)
  if self.StriegelSound ~= nil then
    delete(self.StriegelSound)
    self.StriegelSoundEnabled = false
  end
end
function Striegel:readUpdateStream(streamId, timestamp, connection)
  if connection:getIsServer() then
    self.StriegelHasGroundContact = streamReadBool(streamId)
    self.showFieldNotOwnedWarning = streamReadBool(streamId)
  end
end
function Striegel:writeUpdateStream(streamId, connection, dirtyMask)
  if not connection:getIsServer() then
    streamWriteBool(streamId, self.StriegelHasGroundContact)
    streamWriteBool(streamId, self.showFieldNotOwnedWarning)
  end
end
function Striegel:mouseEvent(posX, posY, isDown, isUp, button)
end
function Striegel:keyEvent(unicode, sym, modifier, isDown)
end
function Striegel:update(dt)
end
function Striegel:updateTick(dt)
  if self:getIsActive() then
    local showFieldNotOwnedWarning = false
    if self.isServer then
      local hasGroundContact = false
      for k, v in pairs(self.contactReportNodes) do
        if v.hasGroundContact then
          hasGroundContact = true
          break
        end
      end
      if not hasGroundContact and self.groundReferenceNode ~= nil then
        local x, y, z = getWorldTranslation(self.groundReferenceNode)
        local terrainHeight = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, x, 0, z)
        if y <= terrainHeight + self.groundReferenceThreshold then
          hasGroundContact = true
        end
      end
      if self.StriegelHasGroundContact ~= hasGroundContact then
        self:raiseDirtyFlags(self.StriegelGroundContactFlag)
        self.StriegelHasGroundContact = hasGroundContact
      end
    end
    local hasGroundContact = self.StriegelHasGroundContact
    local doGroundManipulation = hasGroundContact and (not self.onlyActiveWhenLowered or self:isLowered(false)) and self.startActivationTime <= self.time
    local foldAnimTime = self.foldAnimTime
    if doGroundManipulation and self.isServer then
      local cuttingAreasSend = {}
      for _, cuttingArea in pairs(self.cuttingAreas) do
        if self:getIsAreaActive(cuttingArea) then
          local x, _, z = getWorldTranslation(cuttingArea.start)
          if g_currentMission:getIsFieldOwnedAtWorldPos(x, z) then
            do
              local x1, _, z1 = getWorldTranslation(cuttingArea.width)
              local x2, _, z2 = getWorldTranslation(cuttingArea.height)
              table.insert(cuttingAreasSend, {
                x,
                z,
                x1,
                z1,
                x2,
                z2
              })
            end
          else
            showFieldNotOwnedWarning = true
          end
        end
      end
      if 0 < table.getn(cuttingAreasSend) then
        local limitToField = not self.StriegelLimitToField and self.StriegelForceLimitToField
        local limitGrassDestructionToField = false
        if not g_currentMission:getHasPermission("createFields", self:getOwner()) then
          limitToField = true
          limitGrassDestructionToField = true
        end
        local dx, dy, dz = localDirectionToWorld(self.StriegelDirectionNode, 0, 0, 1)
        local angle = Utils.convertToDensityMapAngle(Utils.getYRotationFromDirection(dx, dz), g_currentMission.terrainDetailAngleMaxValue)
        local realArea = StriegelAreaEvent.runLocally(cuttingAreasSend, limitToField, limitGrassDestructionToField, angle)
        g_server:broadcastEvent(StriegelAreaEvent:new(cuttingAreasSend, limitToField, limitGrassDestructionToField, angle))
        local pixelToSqm = g_currentMission:getFruitPixelsToSqm()
        local sqm = realArea * pixelToSqm
        local ha = sqm / 10000
        g_currentMission.missionStats.hectaresWorkedTotal = g_currentMission.missionStats.hectaresWorkedTotal + ha
        g_currentMission.missionStats.hectaresWorkedSession = g_currentMission.missionStats.hectaresWorkedSession + ha
      end
    end
    if self.isClient then
      for _, ps in pairs(self.groundParticleSystems) do
        local enabled = doGroundManipulation and self.lastSpeed * 3600 > 5
        if enabled and ps.cuttingArea ~= nil and self.cuttingAreas[ps.cuttingArea] ~= nil then
          enabled = self:getIsAreaActive(self.cuttingAreas[ps.cuttingArea])
        end
        if ps.isActive ~= enabled then
          ps.isActive = enabled
          Utils.setEmittingState(ps.ps, ps.isActive)
        end
      end
      if self.StriegelSound ~= nil then
        if doGroundManipulation and self.lastSpeed * 3600 > 3 then
          if not self.StriegelSoundEnabled and self:getIsActiveForSound() then
            playSample(self.StriegelSound, 0, self.StriegelSoundVolume, 0)
            self.StriegelSoundEnabled = true
          end
        elseif self.StriegelSoundEnabled then
          stopSample(self.StriegelSound)
          self.StriegelSoundEnabled = false
        end
      end
      local updateWheelRotatingParts = hasGroundContact
      if not updateWheelRotatingParts then
        for k, v in pairs(self.wheels) do
          if v.hasGroundContact then
            updateWheelRotatingParts = true
            break
          end
        end
      end
      if updateWheelRotatingParts then
        for k, v in pairs(self.speedRotatingParts) do
          if (doGroundManipulation or v.rotateOnGroundContact) and (foldAnimTime == nil or foldAnimTime <= v.foldMaxLimit and foldAnimTime >= v.foldMinLimit) then
            rotate(v.node, v.rotationSpeedScale * self.lastSpeedReal * self.movingDirection * dt, 0, 0)
          end
        end
      end
    end
    local speedLimit = 20
    if self.maxSpeedLevel == 2 then
      speedLimit = 30
    elseif self.maxSpeedLevel == 3 then
      speedLimit = 100
    end
    if doGroundManipulation and self:doCheckSpeedLimit() and speedLimit < self.lastSpeed * 3600 then
      self.speedViolationTimer = self.speedViolationTimer - dt
      if self.isServer and 0 > self.speedViolationTimer and self.attacherVehicle ~= nil then
        self.attacherVehicle:detachImplementByObject(self)
      end
    else
      self.speedViolationTimer = self.speedViolationMaxTime
    end
    if self.isServer and showFieldNotOwnedWarning ~= self.showFieldNotOwnedWarning then
      self.showFieldNotOwnedWarning = showFieldNotOwnedWarning
      self:raiseDirtyFlags(self.StriegelGroundContactFlag)
    end
  end
end
function Striegel:draw()
  if math.abs(self.speedViolationTimer - self.speedViolationMaxTime) > 2 then
    do
      local buttonName = InputBinding.SPEED_LEVEL1
      if self.maxSpeedLevel == 2 then
        buttonName = InputBinding.SPEED_LEVEL2
      elseif self.maxSpeedLevel == 3 then
        buttonName = InputBinding.SPEED_LEVEL3
      end
      g_currentMission:addWarning(g_i18n:getText("Dont_drive_to_fast") .. "\n" .. string.format(g_i18n:getText("Cruise_control_levelN"), tostring(self.maxSpeedLevel)), 0.092, 0.048)
    end
  elseif self.showFieldNotOwnedWarning then
    g_currentMission:addWarning(g_i18n:getText("You_dont_own_this_field"))
  end
end
function Striegel:onAttach(attacherVehicle)
  Striegel.onActivate(self)
  Striegel.addContactReports(self)
  self.startActivationTime = self.time + self.startActivationTimeout
end
function Striegel:onDetach()
  self.StriegelLimitToField = false
  if self.deactivateOnDetach then
    Striegel.onDeactivate(self)
    Striegel.removeContactReports(self)
  else
    Striegel.onDeactivateSounds(self)
  end
end
function Striegel:onEnter(isControlling)
  if isControlling then
    Striegel.onActivate(self)
    Striegel.addContactReports(self)
  end
end
function Striegel:onLeave()
  if self.deactivateOnLeave then
    Striegel.onDeactivate(self)
    Striegel.removeContactReports(self)
  else
    Striegel.onDeactivateSounds(self)
  end
end
function Striegel:onActivate()
end
function Striegel:onDeactivate()
  self.speedViolationTimer = self.speedViolationMaxTime
  self.showFieldNotOwnedWarning = false
  for _, ps in pairs(self.groundParticleSystems) do
    if ps.isActive then
      ps.isActive = false
      Utils.setEmittingState(ps.ps, false)
    end
  end
  Striegel.onDeactivateSounds(self)
end
function Striegel:onDeactivateSounds()
  if self.StriegelSoundEnabled then
    stopSample(self.StriegelSound)
    self.StriegelSoundEnabled = false
  end
end
function Striegel:aiTurnOn()
  self.StriegelLimitToField = true
end
function Striegel:aiTurnOff()
  self.StriegelLimitToField = false
end
function Striegel:addContactReports()
  if not self.StriegelContactReportsActive then
    for k, v in pairs(self.contactReportNodes) do
      addContactReport(v.node, 1.0E-4, "groundContactReport", self)
    end
    self.StriegelContactReportsActive = true
  end
end
function Striegel:removeContactReports()
  if self.StriegelContactReportsActive then
    for k, v in pairs(self.contactReportNodes) do
      removeContactReport(v.node)
      v.hasGroundContact = false
    end
    self.StriegelContactReportsActive = false
  end
end
function Striegel:groundContactReport(objectId, otherObjectId, isStart, normalForce, tangentialForce)
  if otherObjectId == g_currentMission.terrainRootNode then
    local entry = self.contactReportNodes[objectId]
    if entry ~= nil then
      entry.hasGroundContact = isStart or normalForce > 0 or tangentialForce > 0
    end
  end
end
