-- Striegel specialization for mechanical weed management
--
-- only usefull in combination with multiSprayer/Herbicide-Mod by upsidedown
-- check http://www.modhoster.de/mods/multisprayer-herbizid-mod
--
--
-- removes not fully grown weed mechanically
--
-- 99% based on cultivator-specialisation by Stefan Geiger/Giants software
--
-- *** It is explicitely allowed to use this specialisation in your own mods ***
-- for changes please contact upsidedown at modhoster.de
--
-- V1.0:
-- start: 24.04.2014
-- finished: 29.04.2014
-- Author: upsidedown


StriegelAreaEvent = {}
StriegelAreaEvent_mt = Class(StriegelAreaEvent, Event)
InitEventClass(StriegelAreaEvent, "StriegelAreaEvent")
function StriegelAreaEvent:emptyNew()
  local self = Event:new(StriegelAreaEvent_mt)
  return self
end
function StriegelAreaEvent:new(cuttingAreas, limitToField, limitGrassDestructionToField, angle)
  local self = StriegelAreaEvent:emptyNew()
  assert(table.getn(cuttingAreas) > 0)
  self.cuttingAreas = cuttingAreas
  self.limitToField = limitToField
  self.limitGrassDestructionToField = Utils.getNoNil(limitGrassDestructionToField, limitToField)
  self.angle = angle
  return self
end
function StriegelAreaEvent:readStream(streamId, connection)
  local limitToField = streamReadBool(streamId)
  local limitGrassDestructionToField = streamReadBool(streamId)
  local angle
  if streamReadBool(streamId) then
    angle = streamReadUIntN(streamId, g_currentMission.terrainDetailAngleNumChannels)
  end
  local numAreas = streamReadUIntN(streamId, 4)
  local refX = streamReadFloat32(streamId)
  local refY = streamReadFloat32(streamId)
  local values = Utils.readCompressed2DVectors(streamId, refX, refY, numAreas * 3 - 1, 0.01, true)
  for i = 1, numAreas do
    local vi = i - 1
    local x = values[vi * 3 + 1].x
    local z = values[vi * 3 + 1].y
    local x1 = values[vi * 3 + 2].x
    local z1 = values[vi * 3 + 2].y
    local x2 = values[vi * 3 + 3].x
    local z2 = values[vi * 3 + 3].y
    StriegelAreaEvent.updateStriegelArea(x, z, x1, z1, x2, z2, not limitToField, not limitGrassDestructionToField, angle)
  end
end
function StriegelAreaEvent:writeStream(streamId, connection)
  local numAreas = table.getn(self.cuttingAreas)
  streamWriteBool(streamId, self.limitToField)
  streamWriteBool(streamId, self.limitGrassDestructionToField)
  if streamWriteBool(streamId, self.angle ~= nil) then
    streamWriteUIntN(streamId, self.angle, g_currentMission.terrainDetailAngleNumChannels)
  end
  streamWriteUIntN(streamId, numAreas, 4)
  local refX, refY
  local values = {}
  for i = 1, numAreas do
    local d = self.cuttingAreas[i]
    if i == 1 then
      refX = d[1]
      refY = d[2]
      streamWriteFloat32(streamId, d[1])
      streamWriteFloat32(streamId, d[2])
    else
      table.insert(values, {
        x = d[1],
        y = d[2]
      })
    end
    table.insert(values, {
      x = d[3],
      y = d[4]
    })
    table.insert(values, {
      x = d[5],
      y = d[6]
    })
  end
  assert(table.getn(values) == numAreas * 3 - 1)
  Utils.writeCompressed2DVectors(streamId, refX, refY, values, 0.01)
end
function StriegelAreaEvent:run(connection)
  print("Error: Do not run StriegelAreaEvent locally")
end
function StriegelAreaEvent.runLocally(cuttingAreas, limitToField, limitGrassDestructionToField, angle)
  local numAreas = table.getn(cuttingAreas)
  local refX, refY
  local values = {}
  for i = 1, numAreas do
    local d = cuttingAreas[i]
    if i == 1 then
      refX = d[1]
      refY = d[2]
    else
      table.insert(values, {
        x = d[1],
        y = d[2]
      })
    end
    table.insert(values, {
      x = d[3],
      y = d[4]
    })
    table.insert(values, {
      x = d[5],
      y = d[6]
    })
  end
  assert(table.getn(values) == numAreas * 3 - 1)
  local values = Utils.simWriteCompressed2DVectors(refX, refY, values, 0.01, true)
  local areaSum = 0
  for i = 1, numAreas do
    local vi = i - 1
    local x = values[vi * 3 + 1].x
    local z = values[vi * 3 + 1].y
    local x1 = values[vi * 3 + 2].x
    local z1 = values[vi * 3 + 2].y
    local x2 = values[vi * 3 + 3].x
    local z2 = values[vi * 3 + 3].y
    areaSum = areaSum + StriegelAreaEvent.updateStriegelArea(x, z, x1, z1, x2, z2, not limitToField, not limitGrassDestructionToField, angle)
  end
  return areaSum
end


function StriegelAreaEvent.updateStriegelArea(startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ, forced, commonForced, angle)
	local res = 0;
	local detailId = g_currentMission.terrainDetailId;
	local x,z, widthX,widthZ, heightX,heightZ = Utils.getXZWidthAndHeight(detailId, startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ);
	
	if g_currentMission.fruits[FruitUtil.FRUITTYPE_WEED] ~= nil then		
		local id = g_currentMission.fruits[FruitUtil.FRUITTYPE_WEED].id;	
		
		local excludeMask = 0;
		
		setDensityMaskParams(id, "greater", 0, 0, 0, excludeMask);
		setDensityCompareParams(id, "between", 4, 6);
		
		res = setDensityMaskedParallelogram(id, x, z, widthX, widthZ, heightX, heightZ, 0, g_currentMission.numFruitDensityMapChannels, detailId, g_currentMission.terrainDetailTypeFirstChannel, 		g_currentMission.terrainDetailTypeNumChannels, 2);
		
		setDensityCompareParams(id, "greater", -1);
		setDensityMaskParams(id, "greater", 0);		
	end;
	
	
	for index,fruit in pairs(g_currentMission.fruits) do
		if (index ~= FruitUtil.FRUITTYPE_WEED) and fruit.id > 0 then
			local desc = FruitUtil.fruitIndexToDesc[index];
			if desc~= nil then
				if desc.minPreparingGrowthState < 0 then --keine hackfrucht
					--setDensityCompareParams(fruit.id, "equals", gs);
					setDensityCompareParams(fruit.id, "between", desc.minHarvestingGrowthState - 1, desc.maxHarvestingGrowthState + 1)

					local sum = setDensityParallelogram(fruit.id, x, z, widthX, widthZ, heightX, heightZ, 0, g_currentMission.numFruitStateChannels,  desc.cutState + 1);
					if sum>0 then
						--print("destroyed fruit: "..tostring(sum))
					end;
				end;
			end;
		end;
	end;
	
	return res;
end;